module Social
  class Engagement

    def self.for(model_instance)
      url = Url.for(model_instance)
      for_url(url)
    end

    def self.for_url(url)
      begin
        try_to_get_from_cache(url)
      rescue ActiveRecord::RecordNotFound
        social_engagement_from_web = from_web(url)
        social_engagement_from_web.save
        social_engagement_from_web
      end
    end

    def self.from_web(url)
      facebook = Facebook.total_count(url)
      twitter = Twitter.total_count(url)
      SocialEngagement.new(url: url, facebook: facebook, twitter: twitter)
    end

    private

    def self.try_to_get_from_cache(url)
      cached_social_engagement = SocialEngagement.for_url(url)
      if InvalidationPolicy.valid?(cached_social_engagement)
        cached_social_engagement
      else
        social_engagement_from_web = from_web(url)
        cached_social_engagement.update_counts(social_engagement_from_web)
        cached_social_engagement
      end
    end
  end
end
